%{ /* -*- c -*- */
// lexer.l --- a description of the syntax for flex.
// Copyright (C) 1989 Carey Richard Murphey.
// (rich@rice.edu) 5310 Rutherglenn, Houston, TX 77096

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 1, or (at your option)
// any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <stream.h>
#include "parser.h"
#include "Ast.h"
#include "String.Symbolp.AVLMap.h"
#include "String.XPlex.h"
#include "Symbol.h"
#include "Table.XPlex.h"
#include "Context.h"

#include "y.tab.h"
extern "C" double atof (const char *);
#undef ECHO
#define ECHO cerr << yytext
#undef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) { cerr << msg << '\n'; exit (1);}
#define input yyinput
%}
A	[a-zA-Z_]
B	[a-zA-Z0-9_]
D	[0-9]
WS	[ \t\n]+
S	[-+]?
E	([eEdD]{S}{D}+)
%%
{WS}		{ ECHO;}
{D}+		|
{D}+"."{D}*{E}? |
"."{D}+{E}?	|
{D}+{E}		{ ECHO; yylval.valu = atof (yytext); return NUM;}
"//".*$	 	{ ECHO;}
"/*".*"*/"	{ ECHO;}
if		{ ECHO; return IF;}
syntaxof	{ ECHO; return SYNTAXOF;}
while		{ ECHO; return WHILE;}
double		{ ECHO; return DOUBLE_DECL;}
int		{ ECHO; return INT_DECL;}
{A}{B}*		{ ECHO; yylval.idnt = yytext; return IDENT;}
"++"		{ ECHO; return INCR;}
"--"		{ ECHO; return DECR;}
.		{ ECHO; return yytext[0];}
%%
